TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    ADD10_OUT : UINT;
    EQ13_OUT : BOOL;
    SEL15_OUT : UINT;
    AND7_OUT : BOOL;
  END_VAR

  ADD10_OUT := ADD(COUNTER, 1);
  EQ13_OUT := EQ(N, ADD10_OUT);
  SEL15_OUT := SEL(EQ13_OUT, ADD10_OUT, 0);
  COUNTER := SEL15_OUT;
  AND7_OUT := AND(EQ13_OUT, TRIG);
  py_eval(TRIG := AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK


FUNCTION_BLOCK SFC_MASTER
  VAR_INPUT
    INPUT : BOOL;
  END_VAR
  VAR_OUTPUT
    SOSPENDI : BOOL;
    ACCENDI : BOOL;
  END_VAR
  VAR
    R_TRIG1 : R_TRIG;
    R_TRIG2 : R_TRIG;
  END_VAR

  INITIAL_STEP Spento:
    SOSPENDI(N);
    COMPUTE_FUNCTION_BLOCKS(S);
  END_STEP

  ACTION COMPUTE_FUNCTION_BLOCKS:
    R_TRIG1(CLK := INPUT);
    R_TRIG2(CLK := INPUT);
  END_ACTION

  TRANSITION FROM Spento TO Acceso
    := R_TRIG2.Q;
  END_TRANSITION

  STEP Acceso:
    ACCENDI(N);
  END_STEP

  TRANSITION FROM Acceso TO Spento
    := R_TRIG1.Q;
  END_TRANSITION

END_FUNCTION_BLOCK

FUNCTION_BLOCK SFC_ONDA
  VAR_INPUT
    FORZATO : BOOL;
    SOSPESO : BOOL;
  END_VAR
  VAR_OUTPUT
    OUTPUT : BOOL;
  END_VAR

  INITIAL_STEP SPENTO:
  END_STEP

  TRANSITION FROM SPENTO TO ONDA_ON
    := FORZATO;
  END_TRANSITION

  STEP ONDA_ON:
    OUTPUT(N);
  END_STEP

  TRANSITION FROM ONDA_ON TO ONDA_OFF
    := (ONDA_ON.T>=T#4s) AND (NOT SOSPESO);
  END_TRANSITION

  STEP ONDA_OFF:
  END_STEP

  TRANSITION FROM ONDA_OFF TO SPENTO
    := SOSPESO;
  END_TRANSITION

  TRANSITION FROM ONDA_OFF TO ONDA_ON
    := (ONDA_OFF.T>=T#6s) AND (NOT SOSPESO);
  END_TRANSITION

  TRANSITION FROM ONDA_ON TO SPENTO
    := SOSPESO;
  END_TRANSITION

END_FUNCTION_BLOCK

PROGRAM Main
  VAR
    IN : BOOL;
    MASTER : SFC_MASTER;
    OUT : BOOL;
    ONDA : SFC_ONDA;
  END_VAR

  MASTER(INPUT:=IN);
  ONDA(FORZATO:=MASTER.ACCENDI,SOSPESO:=MASTER.SOSPENDI);
  OUT:=ONDA.OUTPUT;
END_PROGRAM


CONFIGURATION Config0

  RESOURCE Res0 ON PLC
    TASK CicloACopiaMassiva(INTERVAL := T#100ms,PRIORITY := 0);
    PROGRAM instance0 WITH CicloACopiaMassiva : Main;
  END_RESOURCE
END_CONFIGURATION
